/*****************************************************************************
 * Copyright (c) 2014-2026 OpenRCT2 developers
 *
 * For a complete list of all authors, please refer to contributors.md
 * Interested in contributing? Visit https://github.com/OpenRCT2/OpenRCT2
 *
 * OpenRCT2 is licensed under the GNU General Public License version 3.
 *****************************************************************************/

#pragma once

#ifdef ENABLE_SCRIPTING

    #include "../../../profiling/Profiling.h"
    #include "../../Duktape.hpp"

namespace OpenRCT2::Scripting
{
    class ScProfiler
    {
    private:
        duk_context* _ctx{};

    public:
        ScProfiler(duk_context* ctx)
            : _ctx(ctx)
        {
        }

    private:
        DukValue getData()
        {
            const auto& data = Profiling::getData();
            duk_push_array(_ctx);
            duk_uarridx_t index = 0;
            for (const auto& f : data)
            {
                DukObject obj(_ctx);
                obj.Set("name", f->getName());
                obj.Set("callCount", f->getCallCount());
                obj.Set("minTime", f->getMinTime());
                obj.Set("maxTime", f->getMaxTime());
                obj.Set("totalTime", f->getTotalTime());
                obj.Set("averageTime", f->getAverageTime());
                obj.Set("parents", GetFunctionIndexArray(data, f->getParents()));
                obj.Set("children", GetFunctionIndexArray(data, f->getChildren()));
                obj.Take().push();
                duk_put_prop_index(_ctx, /* duk stack index */ -2, index);
                index++;
            }
            return DukValue::take_from_stack(_ctx);
        }

        DukValue GetFunctionIndexArray(
            const std::vector<Profiling::Function*>& all, const std::vector<Profiling::Function*>& items)
        {
            duk_push_array(_ctx);
            duk_uarridx_t index = 0;
            for (const auto& item : items)
            {
                auto it = std::find(all.begin(), all.end(), item);
                if (it != all.end())
                {
                    auto value = static_cast<duk_int_t>(std::distance(all.begin(), it));
                    duk_push_int(_ctx, value);
                    duk_put_prop_index(_ctx, /* duk stack index */ -2, index);
                    index++;
                }
            }
            return DukValue::take_from_stack(_ctx);
        }

        void start()
        {
            Profiling::enable();
        }

        void stop()
        {
            Profiling::disable();
        }

        void reset()
        {
            Profiling::resetData();
        }

        bool enabled_get() const
        {
            return Profiling::isEnabled();
        }

    public:
        static void Register(duk_context* ctx)
        {
            dukglue_register_method(ctx, &ScProfiler::getData, "getData");
            dukglue_register_method(ctx, &ScProfiler::start, "start");
            dukglue_register_method(ctx, &ScProfiler::stop, "stop");
            dukglue_register_method(ctx, &ScProfiler::reset, "reset");
            dukglue_register_property(ctx, &ScProfiler::enabled_get, nullptr, "enabled");
        }
    };
} // namespace OpenRCT2::Scripting

#endif
